<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (LiOAuth2, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2016 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */

defined('_JEXEC') or die;

/**
 * LiOAuth2ChannelHelper
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class LiOAuth2ChannelHelper extends ChannelHelper
{
	// 200 * 90% - To be sure
	const MAX_CHARS_TITLE = 180;

	// Max - 10% safety
	const MAX_CHARS_TEXT = 360;

	const MAX_CHARS_DESC = 256;

	// Max - 10% safety
	const MAX_CHARS_COMMENT = 630;

	protected $lioauth2Client = null;

	protected $lioauth2Callback = null;

	protected $consumer_key = null;

	protected $consumer_secret = null;

	protected $access_token = null;

	protected $access_secret = null;

	protected $expires_in = null;

	protected $is_auth = null;

	protected $me = null;

	/**
	 * ChannelHelper.
	 *
	 * @param   object  $channel  Params.
	 */
	public function __construct($channel)
	{
		parent::__construct($channel);

		if ($channel->id)
		{
			$this->consumer_key = $this->channel->params->get('consumer_key');
			$this->consumer_secret = $this->channel->params->get('consumer_secret');
			$this->access_token = $this->channel->params->get('access_token');
			$this->access_secret = $this->channel->params->get('access_secret');

			require_once JPATH_AUTOTWEET_VENDOR . '/autoload.php';
		}
	}

	/**
	 * Internal service functions
	 *
	 * @param   string  $callbackUrl  Param
	 *
	 * @return	object
	 */
	protected function getApiInstance($callbackUrl = null)
	{
		if ((!$this->lioauth2Client) || ($this->lioauth2Callback != $callbackUrl))
		{
			$this->lioauth2Callback = $callbackUrl;

			$this->lioauth2Client = new Extly\Happyr\LinkedIn\LinkedIn(
				$this->consumer_key,
				$this->consumer_secret
			);

			if ($this->access_token)
			{
				$this->lioauth2Client->setAccessToken($this->access_token);
			}
		}

		return $this->lioauth2Client;
	}

	/**
	 * getUser()
	 *
	 * @return	mixed
	 */
	public function getUser()
	{
		if (empty($this->access_token))
		{
			$this->access_token = null;

			return false;
		}

		$api = $this->getApiInstance();
		$response = $api->api('GET', '/v1/people/~:(id,firstName,lastName,headline,publicProfileUrl)');

		if (isset($response['id']))
		{
			$user = json_decode(json_encode($response));
			$url = $user->publicProfileUrl;

			$result = array(
					'status' => true,
					'error_message' => 'Ok!',
					'user' => $user,
					'url' => $url
			);
		}
		else
		{
			$errorMessage = $response['message'] . ' ' . $response['status'];

			$result = array(
					'status' => false,
					'error_message' => $errorMessage
			);

			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::ERROR, $errorMessage);

			// Just in case, it is shown someday
			JFactory::getApplication()->enqueueMessage($errorMessage, 'error');

			// Invalidating access_token
			$this->channel->setToken($this->channel->id, 'access_token', '');
		}

		return $result;
	}

	/**
	 * isAuth()
	 *
	 * @return	mixed
	 */
	public function isAuth()
	{
		if (empty($this->access_token))
		{
			$this->access_token = null;

			return false;
		}

		$result = false;

		try
		{
			$result = $this->getUser();
		}
		catch (Exception $e)
		{
			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());

			// Just in case, it is shown someday
			JFactory::getApplication()->enqueueMessage($e->getMessage(), 'error');

			// Invalidating access_token
			$ch->setToken($this->channel->id, 'access_token', '');
		}

		return $result;
	}

	/**
	 * getAuthorizationUrl
	 *
	 * @return	string
	 */
	public function getAuthorizationUrl()
	{
		if (empty($this->consumer_key))
		{
			return null;
		}

		require_once dirname(__FILE__) . '/../../controllers/lioauth2channels.php';

		$session = JFactory::getSession();
		$session->set('linkedin-authstate', 1);

		$redirectUri = AutotweetControllerLiOAuth2Channels::getCallbackUrl($this->channel->id);
		$this->getApiInstance($redirectUri);

		return $this->lioauth2Client->getLoginUrl(
			[
				'redirect_uri' => $redirectUri,
				'scope' => [
					'r_basicprofile',
					'rw_company_admin',
					'w_share'
				]
			]
		);
	}

	/**
	 * authenticate
	 *
	 * @param   string  $code   Param
	 * @param   string  $state  Param
	 *
	 * @return	bool
	 */
	public function authenticate($code, $state)
	{
		$access_token = $this->getAccessToken($code, $state);

		if ($access_token)
		{
			$now = null;
			$expiresAt = $access_token->getExpiresAt();

			if ($expiresAt)
			{
				$expiresAt = $expiresAt->getTimestamp();
				$now = JFactory::getDate($expiresAt)->toSql();
			}

			$this->saveAccessToken(
					$this->channel->id,
					$access_token->getToken(),
					$expiresAt,
					'',
					$now
			);
		}
		else
		{
			$msg = 'Unable to retrieve access token. (' . $code . ',' . $state . ')';
			JFactory::getApplication()->enqueueMessage($msg, 'error');

			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::INFO, 'LiOAuth2ChannelHelper ' . $msg);

			// Invalidating access_token
			$this->saveAccessToken($this->channel->id);
		}
	}

	/**
	 * getAccessToken
	 *
	 * @param   string  $code   Param
	 * @param   string  $state  Param
	 *
	 * @return	bool
	 */
	public function getAccessToken($code, $state)
	{
		require_once JPATH_ADMINISTRATOR . '/components/com_autotweet/controllers/lioauth2channels.php';

		$redirectUri = AutotweetControllerLiOAuth2Channels::getCallbackUrl($this->channel->id);
		$this->getApiInstance($redirectUri);
		$access_token = $this->lioauth2Client->getAccessToken();

		if (($access_token) && ($access_token->hasToken()))
		{
			return $access_token;
		}
		else
		{
			$this->access_token_error = null;
			$this->access_token_error_description = null;

			if ( ($access_token) && (isset($access_token->error)) && (isset($access_token->error_description)))
			{
				$this->access_token_error = $access_token->error;
				$this->access_token_error_description = $access_token->error_description;
			}
		}

		return null;
	}

	/**
	 * saveAccessToken
	 *
	 * @param   int     $id            Param
	 * @param   string  $token         Param
	 * @param   string  $expires_in    Param
	 * @param   string  $user_id       Param
	 * @param   string  $expires_date  Param
	 *
	 * @return	bool
	 */
	public function saveAccessToken($id, $token = '', $expires_in = '', $user_id = '', $expires_date = '')
	{
		$ch = F0FTable::getAnInstance('Channel', 'AutoTweetTable');

		$ch->setToken($id, 'access_token', $token);
		$ch->setToken($id, 'expires_in', $expires_in);
		$ch->setToken($id, 'user_id', $user_id);
		$ch->setToken($id, 'expires_date', $expires_date);
	}

	/**
	 * includeHashTags
	 *
	 * @return  bool
	 */
	public function includeHashTags()
	{
		return $this->channel->params->get('hashtags', true);
	}

	/**
	 * getSocialUrl
	 *
	 * @param   object  $user  Param
	 *
	 * @return	string
	 */
	public function getSocialUrl($user)
	{
		return $user->publicProfileUrl;
	}

	/**
	 * sendMessage.
	 *
	 * @param   string  $message  Params
	 * @param   object  $data     Params
	 *
	 * @return  boolean
	 */
	public function sendMessage($message, $data)
	{
		$result = array(
			false,
			'Unknown error'
		);

		$title = $data->title;
		$text = $data->fulltext;
		$url = $data->url;
		$image_url = $data->image_url;
		$media_mode = $this->getMediaMode();

		$result = null;
		$content = array();

		// Post message and/or media
		switch ($media_mode)
		{
			case 'attachment':
				$post_attach = true;
				$post_msg = false;
				break;
			case 'both':
				$post_msg = true;
				$post_attach = true;
				break;
			case 'message':
			default:
				$post_msg = true;
				$post_attach = false;
		}

		if (!empty($title))
		{
			$title = TextUtil::truncString($title, self::MAX_CHARS_TITLE);
			$content['content']['title'] = $title;
		}

		if (!empty($text))
		{
			$text = TextUtil::truncString($text, self::MAX_CHARS_TEXT);
			$content['content']['description'] = $text;
		}

		if (!empty($url))
		{
			$content['content']['submitted-url'] = $url;
		}

		if ( ($post_attach) && (!empty($image_url)) )
		{
			$content['content']['submitted-image-url'] = $image_url;
		}

		// Message
		if ($post_msg)
		{
			$content['comment'] = $message;
		}

		$content['visibility'] = array(
			'code' => 'anyone'
		);

		try
		{
			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::INFO, 'LiOAuth2ChannelHelper sendMessage', $content);

			$options = array('json' => $content);

			$api = $this->getApiInstance();
			$response = $api->post('v1/people/~/shares', $options);

			if (isset($response['updateUrl']))
			{
				$result = array(
						true,
						JText::_('COM_AUTOTWEET_OK') . ' - ' . $response['updateUrl']
				);

				return $result;
			}

			if (isset($response['message']))
			{
				$result = array(
						false,
						$response['message'] . ' - ' . $response['status']
				);

				return $result;
			}
		}
		catch (Exception $e)
		{
			$result = array(
					false,
					$e->getMessage()
			);
		}

		return $result;
	}
}
